@@ -116,7 +116,8 @@ validate_ir(Program* program)
          /* check base format */
          Format base_format = instr->format;
          base_format = (Format)((uint32_t)base_format & ~(uint32_t)Format::SDWA);
-         base_format = (Format)((uint32_t)base_format & ~(uint32_t)Format::DPP);
+         base_format = (Format)((uint32_t)base_format & ~(uint32_t)Format::DPP16);
+         base_format = (Format)((uint32_t)base_format & ~(uint32_t)Format::DPP8);
          if ((uint32_t)base_format & (uint32_t)Format::VOP1)
             base_format = Format::VOP1;
          else if ((uint32_t)base_format & (uint32_t)Format::VOP2)
@@ -163,8 +164,34 @@ validate_ir(Program* program)
                check((instr->definitions[0].isFixed() && instr->definitions[0].physReg() == vcc) ||
                         program->chip_class >= GFX9,
                      "SDWA+VOPC definition must be fixed to vcc on GFX8", instr.get());
+            } else {
+               const Definition& def = instr->definitions[0];
+               check(def.bytes() <= 4, "SDWA definitions must not be larger than 4 bytes",
+                     instr.get());
+               check(def.bytes() >= sdwa.dst_sel.size() + sdwa.dst_sel.offset(),
+                     "SDWA definition selection size must be at most definition size", instr.get());
+               check(
+                  sdwa.dst_sel.size() == 1 || sdwa.dst_sel.size() == 2 || sdwa.dst_sel.size() == 4,
+                  "SDWA definition selection size must be 1, 2 or 4 bytes", instr.get());
+               check(sdwa.dst_sel.offset() % sdwa.dst_sel.size() == 0, "Invalid selection offset",
+                     instr.get());
+               check(def.bytes() == 4 || def.bytes() == sdwa.dst_sel.size(),
+                     "SDWA dst_sel size must be definition size for subdword definitions",
+                     instr.get());
+               check(def.bytes() == 4 || sdwa.dst_sel.offset() == 0,
+                     "SDWA dst_sel offset must be 0 for subdword definitions", instr.get());
             }
 
+            for (unsigned i = 0; i < std::min<unsigned>(2, instr->operands.size()); i++) {
+               const Operand& op = instr->operands[i];
+               check(op.bytes() <= 4, "SDWA operands must not be larger than 4 bytes", instr.get());
+               check(op.bytes() >= sdwa.sel[i].size() + sdwa.sel[i].offset(),
+                     "SDWA operand selection size must be at most operand size", instr.get());
+               check(sdwa.sel[i].size() == 1 || sdwa.sel[i].size() == 2 || sdwa.sel[i].size() == 4,
+                     "SDWA operand selection size must be 1, 2 or 4 bytes", instr.get());
+               check(sdwa.sel[i].offset() % sdwa.sel[i].size() == 0, "Invalid selection offset",
+                     instr.get());
+            }
             if (instr->operands.size() >= 3) {
                check(instr->operands[2].isFixed() && instr->operands[2].physReg() == vcc,
                      "3rd operand must be fixed to vcc with SDWA", instr.get());
@@ -192,10 +219,6 @@ validate_ir(Program* program)
                (instr->opcode == aco_opcode::v_mac_f32 && instr->opcode == aco_opcode::v_mac_f16);
 
             check(sdwa_opcodes || feature_mac, "SDWA can't be used with this opcode", instr.get());
-
-            if (instr->definitions[0].regClass().is_subdword())
-               check((sdwa.dst_sel & sdwa_asuint) == (sdwa_isra | instr->definitions[0].bytes()),
-                     "Unexpected SDWA sel for sub-dword definition", instr.get());
          }
 
          /* check opsel */
@@ -213,6 +236,16 @@ validate_ir(Program* program)
             if (instr->definitions[0].regClass().is_subdword() && !instr->definitions[0].isFixed())
                check((vop3.opsel & (1 << 3)) == 0, "Unexpected opsel for sub-dword definition",
                      instr.get());
+         } else if (instr->isVOP3P()) {
+            VOP3P_instruction& vop3p = instr->vop3p();
+            for (unsigned i = 0; i < instr->operands.size(); i++) {
+               if (instr->operands[i].hasRegClass() &&
+                   instr->operands[i].regClass().is_subdword() && !instr->operands[i].isFixed())
+                  check((vop3p.opsel_lo & (1 << i)) == 0 && (vop3p.opsel_hi & (1 << i)) == 0,
+                        "Unexpected opsel for subdword operand", instr.get());
+            }
+            check(instr->definitions[0].regClass() == v1, "VOP3P must have v1 definition",
+                  instr.get());
          }
 
          /* check for undefs */
@@ -234,8 +267,8 @@ validate_ir(Program* program)
          /* check subdword definitions */
          for (unsigned i = 0; i < instr->definitions.size(); i++) {
             if (instr->definitions[i].regClass().is_subdword())
-               check(instr->isPseudo() || instr->definitions[i].bytes() <= 4,
-                     "Only Pseudo instructions can write subdword registers larger than 4 bytes",
+               check(instr->definitions[i].bytes() <= 4 || instr->isPseudo() || instr->isVMEM(),
+                     "Only Pseudo and VMEM instructions can write subdword registers > 4 bytes",
                      instr.get());
          }
 
@@ -273,7 +306,7 @@ validate_ir(Program* program)
                if (instr->isSDWA())
                   scalar_mask = program->chip_class >= GFX9 ? 0x7 : 0x4;
                else if (instr->isDPP())
-                  scalar_mask = 0x0;
+                  scalar_mask = 0x4;
 
                if (instr->isVOPC() || instr->opcode == aco_opcode::v_readfirstlane_b32 ||
                    instr->opcode == aco_opcode::v_readlane_b32 ||
@@ -400,12 +433,20 @@ validate_ir(Program* program)
                for (unsigned i = 0; i < instr->operands.size(); i++) {
                   check(instr->definitions[i].bytes() == instr->operands[i].bytes(),
                         "Operand and Definition size must match", instr.get());
-                  if (instr->operands[i].isTemp())
+                  if (instr->operands[i].isTemp()) {
                      check((instr->definitions[i].getTemp().type() ==
                             instr->operands[i].regClass().type()) ||
                               (instr->definitions[i].getTemp().type() == RegType::vgpr &&
                                instr->operands[i].regClass().type() == RegType::sgpr),
                            "Operand and Definition types do not match", instr.get());
+                     check(instr->definitions[i].regClass().is_linear_vgpr() ==
+                              instr->operands[i].regClass().is_linear_vgpr(),
+                           "Operand and Definition types do not match", instr.get());
+                  } else {
+                     check(!instr->definitions[i].regClass().is_linear_vgpr(),
+                           "Can only copy linear VGPRs into linear VGPRs, not constant/undef",
+                           instr.get());
+                  }
                }
             } else if (instr->opcode == aco_opcode::p_phi) {
                check(instr->operands.size() == block.logical_preds.size(),
@@ -436,22 +477,29 @@ validate_ir(Program* program)
                         instr->operands[0].getTemp().type() == RegType::sgpr,
                      "Can't extract/insert VGPR to SGPR", instr.get());
 
-               if (instr->operands[0].getTemp().type() == RegType::vgpr)
+               if (instr->opcode == aco_opcode::p_insert)
                   check(instr->operands[0].bytes() == instr->definitions[0].bytes(),
-                        "Sizes of operand and definition must match", instr.get());
+                        "Sizes of p_insert data operand and definition must match", instr.get());
 
                if (instr->definitions[0].getTemp().type() == RegType::sgpr)
                   check(instr->definitions.size() >= 2 && instr->definitions[1].isFixed() &&
                            instr->definitions[1].physReg() == scc,
-                        "SGPR extract/insert needs a SCC definition", instr.get());
+                        "SGPR extract/insert needs an SCC definition", instr.get());
 
-               check(instr->operands[2].constantEquals(8) || instr->operands[2].constantEquals(16),
-                     "Size must be 8 or 16", instr.get());
-               check(instr->operands[2].constantValue() < instr->operands[0].getTemp().bytes() * 8u,
-                     "Size must be smaller than source", instr.get());
+               unsigned data_bits = instr->operands[0].getTemp().bytes() * 8u;
+               unsigned op_bits = instr->operands[2].constantValue();
 
-               unsigned comp =
-                  instr->operands[0].bytes() * 8u / MAX2(instr->operands[2].constantValue(), 1);
+               if (instr->opcode == aco_opcode::p_insert) {
+                  check(op_bits == 8 || op_bits == 16, "Size must be 8 or 16", instr.get());
+                  check(op_bits < data_bits, "Size must be smaller than source", instr.get());
+               } else if (instr->opcode == aco_opcode::p_extract) {
+                  check(op_bits == 8 || op_bits == 16 || op_bits == 32,
+                        "Size must be 8 or 16 or 32", instr.get());
+                  check(data_bits >= op_bits, "Can't extract more bits than what the data has.",
+                        instr.get());
+               }
+
+               unsigned comp = data_bits / MAX2(op_bits, 1);
                check(instr->operands[1].constantValue() < comp, "Index must be in-bounds",
                      instr.get());
             }
@@ -505,6 +553,36 @@ validate_ir(Program* program)
                      (instr->operands[3].isTemp() &&
                       instr->operands[3].regClass().type() == RegType::vgpr),
                   "VMEM write data must be vgpr", instr.get());
+
+            const bool d16 = instr->opcode == aco_opcode::buffer_load_dword || // FIXME: used to spill subdword variables
+                             instr->opcode == aco_opcode::buffer_load_ubyte ||
+                             instr->opcode == aco_opcode::buffer_load_sbyte ||
+                             instr->opcode == aco_opcode::buffer_load_ushort ||
+                             instr->opcode == aco_opcode::buffer_load_sshort ||
+                             instr->opcode == aco_opcode::buffer_load_ubyte_d16 ||
+                             instr->opcode == aco_opcode::buffer_load_ubyte_d16_hi ||
+                             instr->opcode == aco_opcode::buffer_load_sbyte_d16 ||
+                             instr->opcode == aco_opcode::buffer_load_sbyte_d16_hi ||
+                             instr->opcode == aco_opcode::buffer_load_short_d16 ||
+                             instr->opcode == aco_opcode::buffer_load_short_d16_hi ||
+                             instr->opcode == aco_opcode::buffer_load_format_d16_x ||
+                             instr->opcode == aco_opcode::buffer_load_format_d16_hi_x ||
+                             instr->opcode == aco_opcode::buffer_load_format_d16_xy ||
+                             instr->opcode == aco_opcode::buffer_load_format_d16_xyz ||
+                             instr->opcode == aco_opcode::buffer_load_format_d16_xyzw ||
+                             instr->opcode == aco_opcode::tbuffer_load_format_d16_x ||
+                             instr->opcode == aco_opcode::tbuffer_load_format_d16_xy ||
+                             instr->opcode == aco_opcode::tbuffer_load_format_d16_xyz ||
+                             instr->opcode == aco_opcode::tbuffer_load_format_d16_xyzw;
+            if (instr->definitions.size()) {
+               check(instr->definitions[0].isTemp() &&
+                        instr->definitions[0].regClass().type() == RegType::vgpr,
+                     "VMEM definitions[0] (VDATA) must be VGPR", instr.get());
+               check(d16 || !instr->definitions[0].regClass().is_subdword(),
+                     "Only D16 opcodes can load subdword values.", instr.get());
+               check(instr->definitions[0].bytes() <= 8 || !d16,
+                     "D16 opcodes can only load up to 8 bytes.", instr.get());
+            }
             break;
          }
          case Format::MIMG: {
@@ -538,10 +616,16 @@ validate_ir(Program* program)
                         instr.get());
                }
             }
-            check(instr->definitions.empty() ||
-                     (instr->definitions[0].isTemp() &&
-                      instr->definitions[0].regClass().type() == RegType::vgpr),
-                  "MIMG definitions[0] (VDATA) must be VGPR", instr.get());
+
+            if (instr->definitions.size()) {
+               check(instr->definitions[0].isTemp() &&
+                        instr->definitions[0].regClass().type() == RegType::vgpr,
+                     "MIMG definitions[0] (VDATA) must be VGPR", instr.get());
+               check(instr->mimg().d16 || !instr->definitions[0].regClass().is_subdword(),
+                     "Only D16 MIMG instructions can load subdword values.", instr.get());
+               check(instr->definitions[0].bytes() <= 8 || !instr->mimg().d16,
+                     "D16 MIMG instructions can only load up to 8 bytes.", instr.get());
+            }
             break;
          }
          case Format::DS: {
@@ -633,6 +717,7 @@ struct Assignment {
    Location defloc;
    Location firstloc;
    PhysReg reg;
+   bool valid;
 };
 
 bool
@@ -680,10 +765,12 @@ validate_subdword_operand(chip_class chip, const aco_ptr<Instruction>& instr, un
       return byte == 0;
    if (instr->isPseudo() && chip >= GFX8)
       return true;
-   if (instr->isSDWA()) {
-      unsigned sel = instr->sdwa().sel[index] & sdwa_asuint;
-      return (sel & sdwa_isra) && (sel & sdwa_rasize) <= op.bytes();
-   }
+   if (instr->isSDWA())
+      return byte + instr->sdwa().sel[index].offset() + instr->sdwa().sel[index].size() <= 4 &&
+             byte % instr->sdwa().sel[index].size() == 0;
+   if (instr->isVOP3P())
+      return ((instr->vop3p().opsel_lo >> index) & 1) == (byte >> 1) &&
+             ((instr->vop3p().opsel_hi >> index) & 1) == (byte >> 1);
    if (byte == 2 && can_use_opsel(chip, instr->opcode, index, 1))
       return true;
 
@@ -707,6 +794,7 @@ validate_subdword_operand(chip_class chip, const aco_ptr<Instruction>& instr, un
       break;
    case aco_opcode::buffer_store_byte_d16_hi:
    case aco_opcode::buffer_store_short_d16_hi:
+   case aco_opcode::buffer_store_format_d16_hi_x:
       if (byte == 2 && index == 3)
          return true;
       break;
@@ -733,14 +821,17 @@ validate_subdword_definition(chip_class chip, const aco_ptr<Instruction>& instr)
 
    if (instr->isPseudo() && chip >= GFX8)
       return true;
-   if (instr->isSDWA() && instr->sdwa().dst_sel == (sdwa_isra | def.bytes()))
-      return true;
+   if (instr->isSDWA())
+      return byte + instr->sdwa().dst_sel.offset() + instr->sdwa().dst_sel.size() <= 4 &&
+             byte % instr->sdwa().dst_sel.size() == 0;
    if (byte == 2 && can_use_opsel(chip, instr->opcode, -1, 1))
       return true;
 
    switch (instr->opcode) {
    case aco_opcode::buffer_load_ubyte_d16_hi:
+   case aco_opcode::buffer_load_sbyte_d16_hi:
    case aco_opcode::buffer_load_short_d16_hi:
+   case aco_opcode::buffer_load_format_d16_hi_x:
    case aco_opcode::flat_load_ubyte_d16_hi:
    case aco_opcode::flat_load_short_d16_hi:
    case aco_opcode::scratch_load_ubyte_d16_hi:
@@ -763,12 +854,28 @@ get_subdword_bytes_written(Program* program, const aco_ptr<Instruction>& instr,
 
    if (instr->isPseudo())
       return chip >= GFX8 ? def.bytes() : def.size() * 4u;
-   if (instr->isSDWA() && instr->sdwa().dst_sel == (sdwa_isra | def.bytes()))
-      return def.bytes();
+   if (instr->isVALU()) {
+      assert(def.bytes() <= 2);
+      if (instr->isSDWA())
+         return instr->sdwa().dst_sel.size();
+
+      if (instr_is_16bit(chip, instr->opcode))
+         return 2;
+
+      return 4;
+   }
+
+   if (instr->isMIMG()) {
+      assert(instr->mimg().d16);
+      return program->dev.sram_ecc_enabled ? def.size() * 4u : def.bytes();
+   }
 
    switch (instr->opcode) {
    case aco_opcode::buffer_load_ubyte_d16:
+   case aco_opcode::buffer_load_sbyte_d16:
    case aco_opcode::buffer_load_short_d16:
+   case aco_opcode::buffer_load_format_d16_x:
+   case aco_opcode::tbuffer_load_format_d16_x:
    case aco_opcode::flat_load_ubyte_d16:
    case aco_opcode::flat_load_short_d16:
    case aco_opcode::scratch_load_ubyte_d16:
@@ -778,7 +885,9 @@ get_subdword_bytes_written(Program* program, const aco_ptr<Instruction>& instr,
    case aco_opcode::ds_read_u8_d16:
    case aco_opcode::ds_read_u16_d16:
    case aco_opcode::buffer_load_ubyte_d16_hi:
+   case aco_opcode::buffer_load_sbyte_d16_hi:
    case aco_opcode::buffer_load_short_d16_hi:
+   case aco_opcode::buffer_load_format_d16_hi_x:
    case aco_opcode::flat_load_ubyte_d16_hi:
    case aco_opcode::flat_load_short_d16_hi:
    case aco_opcode::scratch_load_ubyte_d16_hi:
@@ -787,20 +896,10 @@ get_subdword_bytes_written(Program* program, const aco_ptr<Instruction>& instr,
    case aco_opcode::global_load_short_d16_hi:
    case aco_opcode::ds_read_u8_d16_hi:
    case aco_opcode::ds_read_u16_d16_hi: return program->dev.sram_ecc_enabled ? 4 : 2;
-   case aco_opcode::v_mad_f16:
-   case aco_opcode::v_mad_u16:
-   case aco_opcode::v_mad_i16:
-   case aco_opcode::v_fma_f16:
-   case aco_opcode::v_div_fixup_f16:
-   case aco_opcode::v_interp_p2_f16:
-      if (chip >= GFX9)
-         return 2;
-      break;
-   default: break;
+   case aco_opcode::buffer_load_format_d16_xyz:
+   case aco_opcode::tbuffer_load_format_d16_xyz: return program->dev.sram_ecc_enabled ? 8 : 6;
+   default: return def.size() * 4;
    }
-
-   return MAX2(chip >= GFX10 ? def.bytes() : 4,
-               instr_info.definition_size[(int)instr->opcode] / 8u);
 }
 
 } /* end namespace */
@@ -816,7 +915,7 @@ validate_ra(Program* program)
    std::vector<std::vector<Temp>> phi_sgpr_ops(program->blocks.size());
    uint16_t sgpr_limit = get_addr_sgpr_from_waves(program, program->num_waves);
 
-   std::map<unsigned, Assignment> assignments;
+   std::vector<Assignment> assignments(program->peekAllocationId());
    for (Block& block : program->blocks) {
       Location loc;
       loc.block = &block;
@@ -837,16 +936,16 @@ validate_ra(Program* program)
                continue;
             if (!op.isFixed())
                err |= ra_fail(program, loc, Location(), "Operand %d is not assigned a register", i);
-            if (assignments.count(op.tempId()) && assignments[op.tempId()].reg != op.physReg())
+            if (assignments[op.tempId()].valid && assignments[op.tempId()].reg != op.physReg())
                err |=
-                  ra_fail(program, loc, assignments.at(op.tempId()).firstloc,
+                  ra_fail(program, loc, assignments[op.tempId()].firstloc,
                           "Operand %d has an inconsistent register assignment with instruction", i);
             if ((op.getTemp().type() == RegType::vgpr &&
                  op.physReg().reg_b + op.bytes() > (256 + program->config->num_vgprs) * 4) ||
                 (op.getTemp().type() == RegType::sgpr &&
                  op.physReg() + op.size() > program->config->num_sgprs &&
                  op.physReg() < sgpr_limit))
-               err |= ra_fail(program, loc, assignments.at(op.tempId()).firstloc,
+               err |= ra_fail(program, loc, assignments[op.tempId()].firstloc,
                               "Operand %d has an out-of-bounds register assignment", i);
             if (op.physReg() == vcc && !program->needs_vcc)
                err |= ra_fail(program, loc, Location(),
@@ -856,8 +955,10 @@ validate_ra(Program* program)
                err |= ra_fail(program, loc, Location(), "Operand %d not aligned correctly", i);
             if (!assignments[op.tempId()].firstloc.block)
                assignments[op.tempId()].firstloc = loc;
-            if (!assignments[op.tempId()].defloc.block)
+            if (!assignments[op.tempId()].defloc.block) {
                assignments[op.tempId()].reg = op.physReg();
+               assignments[op.tempId()].valid = true;
+            }
          }
 
          for (unsigned i = 0; i < instr->definitions.size(); i++) {
@@ -868,14 +969,14 @@ validate_ra(Program* program)
                err |=
                   ra_fail(program, loc, Location(), "Definition %d is not assigned a register", i);
             if (assignments[def.tempId()].defloc.block)
-               err |= ra_fail(program, loc, assignments.at(def.tempId()).defloc,
+               err |= ra_fail(program, loc, assignments[def.tempId()].defloc,
                               "Temporary %%%d also defined by instruction", def.tempId());
             if ((def.getTemp().type() == RegType::vgpr &&
                  def.physReg().reg_b + def.bytes() > (256 + program->config->num_vgprs) * 4) ||
                 (def.getTemp().type() == RegType::sgpr &&
                  def.physReg() + def.size() > program->config->num_sgprs &&
                  def.physReg() < sgpr_limit))
-               err |= ra_fail(program, loc, assignments.at(def.tempId()).firstloc,
+               err |= ra_fail(program, loc, assignments[def.tempId()].firstloc,
                               "Definition %d has an out-of-bounds register assignment", i);
             if (def.physReg() == vcc && !program->needs_vcc)
                err |= ra_fail(program, loc, Location(),
@@ -887,6 +988,7 @@ validate_ra(Program* program)
                assignments[def.tempId()].firstloc = loc;
             assignments[def.tempId()].defloc = loc;
             assignments[def.tempId()].reg = def.physReg();
+            assignments[def.tempId()].valid = true;
          }
       }
    }
@@ -898,23 +1000,22 @@ validate_ra(Program* program)
       std::array<unsigned, 2048> regs; /* register file in bytes */
       regs.fill(0);
 
-      std::set<Temp> live;
-      for (unsigned id : live_vars.live_out[block.index])
-         live.insert(Temp(id, program->temp_rc[id]));
+      IDSet live = live_vars.live_out[block.index];
       /* remove killed p_phi sgpr operands */
       for (Temp tmp : phi_sgpr_ops[block.index])
-         live.erase(tmp);
+         live.erase(tmp.id());
 
       /* check live out */
-      for (Temp tmp : live) {
-         PhysReg reg = assignments.at(tmp.id()).reg;
+      for (unsigned id : live) {
+         Temp tmp(id, program->temp_rc[id]);
+         PhysReg reg = assignments[id].reg;
          for (unsigned i = 0; i < tmp.bytes(); i++) {
             if (regs[reg.reg_b + i]) {
                err |= ra_fail(program, loc, Location(),
                               "Assignment of element %d of %%%d already taken by %%%d in live-out",
-                              i, tmp.id(), regs[reg.reg_b + i]);
+                              i, id, regs[reg.reg_b + i]);
             }
-            regs[reg.reg_b + i] = tmp.id();
+            regs[reg.reg_b + i] = id;
          }
       }
       regs.fill(0);
@@ -925,7 +1026,7 @@ validate_ra(Program* program)
          /* check killed p_phi sgpr operands */
          if (instr->opcode == aco_opcode::p_logical_end) {
             for (Temp tmp : phi_sgpr_ops[block.index]) {
-               PhysReg reg = assignments.at(tmp.id()).reg;
+               PhysReg reg = assignments[tmp.id()].reg;
                for (unsigned i = 0; i < tmp.bytes(); i++) {
                   if (regs[reg.reg_b + i])
                      err |= ra_fail(
@@ -933,14 +1034,14 @@ validate_ra(Program* program)
                         "Assignment of element %d of %%%d already taken by %%%d in live-out", i,
                         tmp.id(), regs[reg.reg_b + i]);
                }
-               live.emplace(tmp);
+               live.insert(tmp.id());
             }
          }
 
          for (const Definition& def : instr->definitions) {
             if (!def.isTemp())
                continue;
-            live.erase(def.getTemp());
+            live.erase(def.tempId());
          }
 
          /* don't count phi operands as live-in, since they are actually
@@ -949,15 +1050,16 @@ validate_ra(Program* program)
             for (const Operand& op : instr->operands) {
                if (!op.isTemp())
                   continue;
-               live.insert(op.getTemp());
+               live.insert(op.tempId());
             }
          }
       }
 
-      for (Temp tmp : live) {
-         PhysReg reg = assignments.at(tmp.id()).reg;
+      for (unsigned id : live) {
+         Temp tmp(id, program->temp_rc[id]);
+         PhysReg reg = assignments[id].reg;
          for (unsigned i = 0; i < tmp.bytes(); i++)
-            regs[reg.reg_b + i] = tmp.id();
+            regs[reg.reg_b + i] = id;
       }
 
       for (aco_ptr<Instruction>& instr : block.instructions) {
@@ -966,7 +1068,7 @@ validate_ra(Program* program)
          /* remove killed p_phi operands from regs */
          if (instr->opcode == aco_opcode::p_logical_end) {
             for (Temp tmp : phi_sgpr_ops[block.index]) {
-               PhysReg reg = assignments.at(tmp.id()).reg;
+               PhysReg reg = assignments[tmp.id()].reg;
                for (unsigned i = 0; i < tmp.bytes(); i++)
                   regs[reg.reg_b + i] = 0;
             }
@@ -988,11 +1090,11 @@ validate_ra(Program* program)
             if (!def.isTemp())
                continue;
             Temp tmp = def.getTemp();
-            PhysReg reg = assignments.at(tmp.id()).reg;
+            PhysReg reg = assignments[tmp.id()].reg;
             for (unsigned j = 0; j < tmp.bytes(); j++) {
                if (regs[reg.reg_b + j])
                   err |= ra_fail(
-                     program, loc, assignments.at(regs[reg.reg_b + j]).defloc,
+                     program, loc, assignments[regs[reg.reg_b + j]].defloc,
                      "Assignment of element %d of %%%d already taken by %%%d from instruction", i,
                      tmp.id(), regs[reg.reg_b + j]);
                regs[reg.reg_b + j] = tmp.id();
@@ -1004,7 +1106,7 @@ validate_ra(Program* program)
                for (unsigned j = reg.byte() & ~(written - 1); j < written; j++) {
                   unsigned written_reg = reg.reg() * 4u + j;
                   if (regs[written_reg] && regs[written_reg] != def.tempId())
-                     err |= ra_fail(program, loc, assignments.at(regs[written_reg]).defloc,
+                     err |= ra_fail(program, loc, assignments[regs[written_reg]].defloc,
                                     "Assignment of element %d of %%%d overwrites the full register "
                                     "taken by %%%d from instruction",
                                     i, tmp.id(), regs[written_reg]);
