@@ -70,6 +70,11 @@ struct radv_meta_saved_state {
    bool depth_bounds_test_enable;
    bool stencil_test_enable;
 
+   struct {
+      uint32_t front;
+      uint32_t back;
+   } stencil_write_mask;
+
    struct {
       struct {
          VkStencilOp fail_op;
@@ -86,6 +91,11 @@ struct radv_meta_saved_state {
       } back;
    } stencil_op;
 
+   struct {
+      uint32_t front;
+      uint32_t back;
+   } stencil_reference;
+
    struct {
       VkExtent2D size;
       VkFragmentShadingRateCombinerOpKHR combiner_ops[2];
