@@ -27,6 +27,8 @@
 #ifndef H_ETNAVIV_SHADER
 #define H_ETNAVIV_SHADER
 
+#include "mesa/main/config.h"
+#include "nir.h"
 #include "pipe/p_state.h"
 #include "util/disk_cache.h"
 
@@ -46,17 +48,28 @@ struct etna_shader_key
          unsigned frag_rb_swap : 1;
          /* do we need to invert front facing value? */
          unsigned front_ccw : 1;
+         /* do we need to replace glTexCoord.xy ? */
+         unsigned sprite_coord_enable : MAX_TEXTURE_COORD_UNITS;
+         unsigned sprite_coord_yinvert : 1;
+         /* do we need to lower sample_tex_compare */
+         unsigned has_sample_tex_compare : 1;
       };
       uint32_t global;
    };
+
+   int num_texture_states;
+   nir_lower_tex_shadow_swizzle tex_swizzle[PIPE_MAX_SHADER_SAMPLER_VIEWS];
+   enum compare_func tex_compare_func[PIPE_MAX_SHADER_SAMPLER_VIEWS];
 };
 
 static inline bool
 etna_shader_key_equal(struct etna_shader_key *a, struct etna_shader_key *b)
 {
-   STATIC_ASSERT(sizeof(struct etna_shader_key) <= sizeof(a->global));
-
-   return a->global == b->global;
+   /* slow-path if we need to check tex_{swizzle,compare_func} */
+   if (unlikely(a->has_sample_tex_compare || b->has_sample_tex_compare))
+      return memcmp(a, b, sizeof(struct etna_shader_key)) == 0;
+   else
+      return a->global == b->global;
 }
 
 struct etna_shader {
