@@ -38,6 +38,7 @@
 #include "freedreno_state.h"
 #include "freedreno_texture.h"
 #include "freedreno_util.h"
+#include "util/u_trace_gallium.h"
 
 static void
 fd_context_flush(struct pipe_context *pctx, struct pipe_fence_handle **fencep,
@@ -244,7 +245,7 @@ fd_emit_string_marker(struct pipe_context *pctx, const char *string,
 
    fd_batch_needs_flush(batch);
 
-   if (ctx->screen->gpu_id >= 500) {
+   if (ctx->screen->gen >= 5) {
       fd_emit_string5(batch->draw, string, len);
    } else {
       fd_emit_string(batch->draw, string, len);
@@ -368,9 +369,6 @@ fd_context_destroy(struct pipe_context *pctx)
       if (ctx->clear_rs_state[i])
          pctx->delete_rasterizer_state(pctx, ctx->clear_rs_state[i]);
 
-   if (ctx->primconvert)
-      util_primconvert_destroy(ctx->primconvert);
-
    slab_destroy_child(&ctx->transfer_pool);
    slab_destroy_child(&ctx->transfer_pool_unsync);
 
@@ -432,11 +430,6 @@ fd_get_device_reset_status(struct pipe_context *pctx)
    int global_faults = fd_get_reset_count(ctx, false);
    enum pipe_reset_status status;
 
-   /* Not called in driver thread, but threaded_context syncs
-    * before calling this:
-    */
-   fd_context_access_begin(ctx);
-
    if (context_faults != ctx->context_reset_count) {
       status = PIPE_GUILTY_CONTEXT_RESET;
    } else if (global_faults != ctx->global_reset_count) {
@@ -448,36 +441,36 @@ fd_get_device_reset_status(struct pipe_context *pctx)
    ctx->context_reset_count = context_faults;
    ctx->global_reset_count = global_faults;
 
-   fd_context_access_end(ctx);
-
    return status;
 }
 
 static void
-fd_trace_record_ts(struct u_trace *ut, struct pipe_resource *timestamps,
-                   unsigned idx)
+fd_trace_record_ts(struct u_trace *ut, void *cs, void *timestamps,
+                   unsigned idx, bool end_of_pipe)
 {
    struct fd_batch *batch = container_of(ut, struct fd_batch, trace);
-   struct fd_ringbuffer *ring = batch->nondraw ? batch->draw : batch->gmem;
+   struct fd_ringbuffer *ring = cs;
+   struct pipe_resource *buffer = timestamps;
 
    if (ring->cur == batch->last_timestamp_cmd) {
-      uint64_t *ts = fd_bo_map(fd_resource(timestamps)->bo);
+      uint64_t *ts = fd_bo_map(fd_resource(buffer)->bo);
       ts[idx] = U_TRACE_NO_TIMESTAMP;
       return;
    }
 
    unsigned ts_offset = idx * sizeof(uint64_t);
-   batch->ctx->record_timestamp(ring, fd_resource(timestamps)->bo, ts_offset);
+   batch->ctx->record_timestamp(ring, fd_resource(buffer)->bo, ts_offset);
    batch->last_timestamp_cmd = ring->cur;
 }
 
 static uint64_t
 fd_trace_read_ts(struct u_trace_context *utctx,
-                 struct pipe_resource *timestamps, unsigned idx)
+                 void *timestamps, unsigned idx, void *flush_data)
 {
    struct fd_context *ctx =
       container_of(utctx, struct fd_context, trace_context);
-   struct fd_bo *ts_bo = fd_resource(timestamps)->bo;
+   struct pipe_resource *buffer = timestamps;
+   struct fd_bo *ts_bo = fd_resource(buffer)->bo;
 
    /* Only need to stall on results for the first entry: */
    if (idx == 0) {
@@ -500,6 +493,12 @@ fd_trace_read_ts(struct u_trace_context *utctx,
    return ctx->ts_to_ns(ts[idx]);
 }
 
+static void
+fd_trace_delete_flush_data(struct u_trace_context *utctx, void *flush_data)
+{
+   /* We don't use flush_data at the moment. */
+}
+
 /* TODO we could combine a few of these small buffers (solid_vbuf,
  * blit_texcoord_vbuf, and vsc_size_mem, into a single buffer and
  * save a tiny bit of memory
@@ -509,7 +508,7 @@ static struct pipe_resource *
 create_solid_vertexbuf(struct pipe_context *pctx)
 {
    static const float init_shader_const[] = {
-      -1.000000, +1.000000, +1.000000, +1.000000, -1.000000, +1.000000,
+      -1.000000f, +1.000000f, +1.000000f, +1.000000f, -1.000000f, +1.000000f,
    };
    struct pipe_resource *prsc =
       pipe_buffer_create(pctx->screen, PIPE_BIND_CUSTOM, PIPE_USAGE_IMMUTABLE,
@@ -583,13 +582,12 @@ fd_context_cleanup_common_vbos(struct fd_context *ctx)
 
 struct pipe_context *
 fd_context_init(struct fd_context *ctx, struct pipe_screen *pscreen,
-                const uint8_t *primtypes, void *priv,
-                unsigned flags) disable_thread_safety_analysis
+                void *priv, unsigned flags)
+   disable_thread_safety_analysis
 {
    struct fd_screen *screen = fd_screen(pscreen);
    struct pipe_context *pctx;
    unsigned prio = 1;
-   int i;
 
    /* lower numerical value == higher priority: */
    if (FD_DBG(HIPRIO))
@@ -605,6 +603,7 @@ fd_context_init(struct fd_context *ctx, struct pipe_screen *pscreen,
    if (FD_DBG(BSTAT) || FD_DBG(MSGS))
       ctx->stats_users++;
 
+   ctx->flags = flags;
    ctx->screen = screen;
    ctx->pipe = fd_pipe_new2(screen->dev, FD_PIPE_3D, prio);
 
@@ -615,12 +614,6 @@ fd_context_init(struct fd_context *ctx, struct pipe_screen *pscreen,
       ctx->global_reset_count = fd_get_reset_count(ctx, false);
    }
 
-   ctx->primtypes = primtypes;
-   ctx->primtype_mask = 0;
-   for (i = 0; i <= PIPE_PRIM_MAX; i++)
-      if (primtypes[i])
-         ctx->primtype_mask |= (1 << i);
-
    simple_mtx_init(&ctx->gmem_lock, mtx_plain);
 
    /* need some sane default in case gallium frontends don't
@@ -660,10 +653,6 @@ fd_context_init(struct fd_context *ctx, struct pipe_screen *pscreen,
    if (!ctx->blitter)
       goto fail;
 
-   ctx->primconvert = util_primconvert_create(pctx, ctx->primtype_mask);
-   if (!ctx->primconvert)
-      goto fail;
-
    list_inithead(&ctx->hw_active_queries);
    list_inithead(&ctx->acc_active_queries);
 
@@ -674,8 +663,10 @@ fd_context_init(struct fd_context *ctx, struct pipe_screen *pscreen,
 
    ctx->current_scissor = &ctx->disabled_scissor;
 
-   u_trace_context_init(&ctx->trace_context, pctx, fd_trace_record_ts,
-                        fd_trace_read_ts);
+   u_trace_pipe_context_init(&ctx->trace_context, pctx,
+                             fd_trace_record_ts,
+                             fd_trace_read_ts,
+                             fd_trace_delete_flush_data);
 
    fd_autotune_init(&ctx->autotune, screen->dev);
 
@@ -701,15 +692,15 @@ fd_context_init_tc(struct pipe_context *pctx, unsigned flags)
    struct pipe_context *tc = threaded_context_create(
       pctx, &ctx->screen->transfer_pool,
       fd_replace_buffer_storage,
-      fd_fence_create_unflushed,
-      fd_resource_busy,
-      false,
+      &(struct threaded_context_options){
+         .create_fence = fd_fence_create_unflushed,
+         .is_resource_busy = fd_resource_busy,
+         .unsynchronized_get_device_reset_status = true,
+      },
       &ctx->tc);
 
-   uint64_t total_ram;
-   if (tc && tc != pctx && os_get_total_physical_memory(&total_ram)) {
-      ((struct threaded_context *)tc)->bytes_mapped_limit = total_ram / 16;
-   }
+   if (tc && tc != pctx)
+      threaded_context_init_bytes_mapped_limit((struct threaded_context *)tc, 16);
 
    return tc;
 }
