@@ -1,8 +1,8 @@
 /**************************************************************************
- * 
+ *
  * Copyright 2008 VMware, Inc.
  * All Rights Reserved.
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the
  * "Software"), to deal in the Software without restriction, including
@@ -10,11 +10,11 @@
  * distribute, sub license, and/or sell copies of the Software, and to
  * permit persons to whom the Software is furnished to do so, subject to
  * the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice (including the
  * next paragraph) shall be included in all copies or substantial portions
  * of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
@@ -22,33 +22,31 @@
  * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
  * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- * 
+ *
  **************************************************************************/
 
-
+#include "compiler/nir/nir.h"
 #include "draw/draw_context.h"
-#include "util/os_misc.h"
 #include "util/format/u_format.h"
 #include "util/format/u_format_s3tc.h"
+#include "util/os_misc.h"
 #include "util/u_inlines.h"
 #include "util/u_memory.h"
 #include "util/u_screen.h"
 #include "util/u_string.h"
 
-#include "i915_reg.h"
-#include "i915_debug.h"
 #include "i915_context.h"
-#include "i915_screen.h"
-#include "i915_resource.h"
-#include "i915_winsys.h"
+#include "i915_debug.h"
 #include "i915_public.h"
-
+#include "i915_reg.h"
+#include "i915_resource.h"
+#include "i915_screen.h"
+#include "i915_winsys.h"
 
 /*
  * Probe functions
  */
 
-
 static const char *
 i915_get_vendor(struct pipe_screen *screen)
 {
@@ -107,29 +105,221 @@ i915_get_name(struct pipe_screen *screen)
    return buffer;
 }
 
+static const nir_shader_compiler_options i915_compiler_options = {
+   .fdot_replicates = true,
+   .fuse_ffma32 = true,
+   .lower_bitops = true, /* required for !CAP_INTEGERS nir_to_tgsi */
+   .lower_extract_byte = true,
+   .lower_extract_word = true,
+   .lower_fdiv = true,
+   .lower_fdph = true,
+   .lower_flrp32 = true,
+   .lower_fmod = true,
+   .lower_rotate = true,
+   .lower_uniforms_to_ubo = true,
+   .lower_vector_cmp = true,
+   .use_interpolated_input_intrinsics = true,
+   .force_indirect_unrolling = ~0,
+};
+
+static const struct nir_shader_compiler_options gallivm_nir_options = {
+   .fdot_replicates = true,
+   .lower_bitops = true, /* required for !CAP_INTEGERS nir_to_tgsi */
+   .lower_scmp = true,
+   .lower_flrp32 = true,
+   .lower_flrp64 = true,
+   .lower_fsat = true,
+   .lower_bitfield_insert_to_shifts = true,
+   .lower_bitfield_extract_to_shifts = true,
+   .lower_fdph = true,
+   .lower_ffma16 = true,
+   .lower_ffma32 = true,
+   .lower_ffma64 = true,
+   .lower_fmod = true,
+   .lower_hadd = true,
+   .lower_uadd_sat = true,
+   .lower_iadd_sat = true,
+   .lower_ldexp = true,
+   .lower_pack_snorm_2x16 = true,
+   .lower_pack_snorm_4x8 = true,
+   .lower_pack_unorm_2x16 = true,
+   .lower_pack_unorm_4x8 = true,
+   .lower_pack_half_2x16 = true,
+   .lower_pack_split = true,
+   .lower_unpack_snorm_2x16 = true,
+   .lower_unpack_snorm_4x8 = true,
+   .lower_unpack_unorm_2x16 = true,
+   .lower_unpack_unorm_4x8 = true,
+   .lower_unpack_half_2x16 = true,
+   .lower_extract_byte = true,
+   .lower_extract_word = true,
+   .lower_rotate = true,
+   .lower_uadd_carry = true,
+   .lower_usub_borrow = true,
+   .lower_mul_2x32_64 = true,
+   .lower_ifind_msb = true,
+   .max_unroll_iterations = 32,
+   .use_interpolated_input_intrinsics = true,
+   .lower_cs_local_index_from_id = true,
+   .lower_uniforms_to_ubo = true,
+   .lower_vector_cmp = true,
+   .lower_device_index_to_zero = true,
+   /* .support_16bit_alu = true, */
+};
+
+static const void *
+i915_get_compiler_options(struct pipe_screen *pscreen, enum pipe_shader_ir ir,
+                          enum pipe_shader_type shader)
+{
+   assert(ir == PIPE_SHADER_IR_NIR);
+   if (shader == PIPE_SHADER_FRAGMENT)
+      return &i915_compiler_options;
+   else
+      return &gallivm_nir_options;
+}
+
+static void
+i915_optimize_nir(struct nir_shader *s)
+{
+   bool progress;
+
+   do {
+      progress = false;
+
+      NIR_PASS_V(s, nir_lower_vars_to_ssa);
+
+      NIR_PASS(progress, s, nir_copy_prop);
+      NIR_PASS(progress, s, nir_opt_algebraic);
+      NIR_PASS(progress, s, nir_opt_constant_folding);
+      NIR_PASS(progress, s, nir_opt_remove_phis);
+      NIR_PASS(progress, s, nir_opt_conditional_discard);
+      NIR_PASS(progress, s, nir_opt_dce);
+      NIR_PASS(progress, s, nir_opt_dead_cf);
+      NIR_PASS(progress, s, nir_opt_cse);
+      NIR_PASS(progress, s, nir_opt_find_array_copies);
+      NIR_PASS(progress, s, nir_opt_if, true);
+      NIR_PASS(progress, s, nir_opt_peephole_select, ~0 /* flatten all IFs. */,
+               true, true);
+      NIR_PASS(progress, s, nir_opt_algebraic);
+      NIR_PASS(progress, s, nir_opt_constant_folding);
+      NIR_PASS(progress, s, nir_opt_shrink_vectors, true);
+      NIR_PASS(progress, s, nir_opt_trivial_continues);
+      NIR_PASS(progress, s, nir_opt_undef);
+      NIR_PASS(progress, s, nir_opt_loop_unroll);
+
+   } while (progress);
+
+   NIR_PASS(progress, s, nir_remove_dead_variables, nir_var_function_temp,
+            NULL);
+}
+
+static char *i915_check_control_flow(nir_shader *s)
+{
+   if (s->info.stage == MESA_SHADER_FRAGMENT) {
+      nir_function_impl *impl = nir_shader_get_entrypoint(s);
+      nir_block *first = nir_start_block(impl);
+      nir_cf_node *next = nir_cf_node_next(&first->cf_node);
+
+      if (next) {
+         switch (next->type) {
+         case nir_cf_node_if:
+            return "if/then statements not supported by i915 fragment shaders, should have been flattened by peephole_select.";
+         case nir_cf_node_loop:
+            return "looping not supported i915 fragment shaders, all loops must be statically unrollable.";
+         default:
+            return "Unknown control flow type";
+         }
+      }
+   }
+
+   return NULL;
+}
+
+static char *
+i915_finalize_nir(struct pipe_screen *pscreen, void *nir)
+{
+   nir_shader *s = nir;
+
+   if (s->info.stage == MESA_SHADER_FRAGMENT)
+      i915_optimize_nir(s);
+
+   /* st_program.c's parameter list optimization requires that future nir
+    * variants don't reallocate the uniform storage, so we have to remove
+    * uniforms that occupy storage.  But we don't want to remove samplers,
+    * because they're needed for YUV variant lowering.
+    */
+   nir_remove_dead_derefs(s);
+   nir_foreach_uniform_variable_safe(var, s)
+   {
+      if (var->data.mode == nir_var_uniform &&
+          (glsl_type_get_image_count(var->type) ||
+           glsl_type_get_sampler_count(var->type)))
+         continue;
+
+      exec_node_remove(&var->node);
+   }
+   nir_validate_shader(s, "after uniform var removal");
+
+   nir_sweep(s);
+
+   char *msg = i915_check_control_flow(s);
+   if (msg)
+      return strdup(msg);
+
+   return NULL;
+}
+
 static int
-i915_get_shader_param(struct pipe_screen *screen,
-                      enum pipe_shader_type shader,
+i915_get_shader_param(struct pipe_screen *screen, enum pipe_shader_type shader,
                       enum pipe_shader_cap cap)
 {
-   switch(shader) {
+   switch (cap) {
+   case PIPE_SHADER_CAP_PREFERRED_IR:
+      return PIPE_SHADER_IR_NIR;
+   case PIPE_SHADER_CAP_SUPPORTED_IRS:
+      return (1 << PIPE_SHADER_IR_NIR) | (1 << PIPE_SHADER_IR_TGSI);
+
+   case PIPE_SHADER_CAP_INTEGERS:
+      /* mesa/st requires that this cap is the same across stages, and the FS
+       * can't do ints.
+       */
+      return 0;
+
+   /* i915 can't do these, and even if gallivm NIR can we call nir_to_tgsi
+    * manually and TGSI can't.
+    */
+   case PIPE_SHADER_CAP_INT16:
+   case PIPE_SHADER_CAP_FP16:
+   case PIPE_SHADER_CAP_FP16_DERIVATIVES:
+   case PIPE_SHADER_CAP_FP16_CONST_BUFFERS:
+      return 0;
+
+   case PIPE_SHADER_CAP_INDIRECT_TEMP_ADDR:
+      /* While draw could normally handle this for the VS, the NIR lowering
+       * to regs can't handle our non-native-integers, so we have to lower to
+       * if ladders.
+       */
+      return 0;
+
+   default:
+      break;
+   }
+
+   switch (shader) {
    case PIPE_SHADER_VERTEX:
       switch (cap) {
       case PIPE_SHADER_CAP_MAX_TEXTURE_SAMPLERS:
       case PIPE_SHADER_CAP_MAX_SAMPLER_VIEWS:
-         if (debug_get_bool_option("DRAW_USE_LLVM", TRUE))
-            return PIPE_MAX_SAMPLERS;
-         else
-            return 0;
+         return 0;
       case PIPE_SHADER_CAP_MAX_SHADER_BUFFERS:
       case PIPE_SHADER_CAP_MAX_SHADER_IMAGES:
          return 0;
-       default:
+      default:
          return draw_get_shader_param(shader, cap);
       }
    case PIPE_SHADER_FRAGMENT:
       /* XXX: some of these are just shader model 2.0 values, fix this! */
-      switch(cap) {
+      switch (cap) {
       case PIPE_SHADER_CAP_MAX_INSTRUCTIONS:
          return I915_MAX_ALU_INSN + I915_MAX_TEX_INSN;
       case PIPE_SHADER_CAP_MAX_ALU_INSTRUCTIONS:
@@ -137,7 +327,7 @@ i915_get_shader_param(struct pipe_screen *screen,
       case PIPE_SHADER_CAP_MAX_TEX_INSTRUCTIONS:
          return I915_MAX_TEX_INSN;
       case PIPE_SHADER_CAP_MAX_TEX_INDIRECTIONS:
-         return 8;
+         return 4;
       case PIPE_SHADER_CAP_MAX_CONTROL_FLOW_DEPTH:
          return 0;
       case PIPE_SHADER_CAP_MAX_INPUTS:
@@ -149,7 +339,8 @@ i915_get_shader_param(struct pipe_screen *screen,
       case PIPE_SHADER_CAP_MAX_CONST_BUFFERS:
          return 1;
       case PIPE_SHADER_CAP_MAX_TEMPS:
-         return 12; /* XXX: 12 -> 32 ? */
+         /* 16 inter-phase temps, 3 intra-phase temps.  i915c reported 16. too. */
+         return 16;
       case PIPE_SHADER_CAP_TGSI_CONT_SUPPORTED:
       case PIPE_SHADER_CAP_TGSI_SQRT_SUPPORTED:
          return 0;
@@ -157,13 +348,9 @@ i915_get_shader_param(struct pipe_screen *screen,
       case PIPE_SHADER_CAP_INDIRECT_OUTPUT_ADDR:
       case PIPE_SHADER_CAP_INDIRECT_TEMP_ADDR:
       case PIPE_SHADER_CAP_INDIRECT_CONST_ADDR:
-         return 1;
       case PIPE_SHADER_CAP_SUBROUTINES:
          return 0;
-      case PIPE_SHADER_CAP_INTEGERS:
       case PIPE_SHADER_CAP_INT64_ATOMICS:
-      case PIPE_SHADER_CAP_FP16:
-      case PIPE_SHADER_CAP_FP16_DERIVATIVES:
       case PIPE_SHADER_CAP_INT16:
       case PIPE_SHADER_CAP_GLSL_16BIT_CONSTS:
          return 0;
@@ -178,9 +365,11 @@ i915_get_shader_param(struct pipe_screen *screen,
       case PIPE_SHADER_CAP_MAX_SHADER_BUFFERS:
       case PIPE_SHADER_CAP_MAX_SHADER_IMAGES:
       case PIPE_SHADER_CAP_LOWER_IF_THRESHOLD:
-      case PIPE_SHADER_CAP_PREFERRED_IR:
       case PIPE_SHADER_CAP_TGSI_SKIP_MERGE_REGISTERS:
+      case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTERS:
+      case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTER_BUFFERS:
          return 0;
+
       case PIPE_SHADER_CAP_MAX_UNROLL_ITERATIONS_HINT:
          return 32;
       default:
@@ -191,7 +380,6 @@ i915_get_shader_param(struct pipe_screen *screen,
    default:
       return 0;
    }
-
 }
 
 static int
@@ -213,9 +401,10 @@ i915_get_param(struct pipe_screen *screen, enum pipe_cap cap)
    case PIPE_CAP_VERTEX_COLOR_CLAMPED:
    case PIPE_CAP_USER_VERTEX_BUFFERS:
    case PIPE_CAP_MIXED_COLOR_DEPTH_BITS:
+   case PIPE_CAP_TGSI_TEXCOORD:
       return 1;
 
-   case PIPE_CAP_PREFER_BLIT_BASED_TEXTURE_TRANSFER:
+   case PIPE_CAP_TEXTURE_TRANSFER_MODES:
    case PIPE_CAP_PCI_GROUP:
    case PIPE_CAP_PCI_BUS:
    case PIPE_CAP_PCI_DEVICE:
@@ -226,6 +415,12 @@ i915_get_param(struct pipe_screen *screen, enum pipe_cap cap)
    case PIPE_CAP_ALLOW_MAPPED_BUFFERS_DURING_EXECUTION:
       return 0;
 
+   case PIPE_CAP_SHAREABLE_SHADERS:
+      /* Can't expose shareable shaders because the draw shaders reference the
+       * draw module's state, which is per-context.
+       */
+      return 0;
+
    case PIPE_CAP_MAX_GS_INVOCATIONS:
       return 32;
 
@@ -245,10 +440,6 @@ i915_get_param(struct pipe_screen *screen, enum pipe_cap cap)
    case PIPE_CAP_CONSTANT_BUFFER_OFFSET_ALIGNMENT:
       return 16;
 
-   /* Features we can lie about (boolean caps). */
-   case PIPE_CAP_OCCLUSION_QUERY:
-      return is->debug.lie ? 1 : 0;
-
    /* Texturing. */
    case PIPE_CAP_MAX_TEXTURE_2D_SIZE:
       return 1 << (I915_MAX_TEXTURE_2D_LEVELS - 1);
@@ -273,6 +464,9 @@ i915_get_param(struct pipe_screen *screen, enum pipe_cap cap)
    case PIPE_CAP_MAX_VARYINGS:
       return 10;
 
+   case PIPE_CAP_NIR_IMAGES_AS_DEREF:
+      return 0;
+
    case PIPE_CAP_VENDOR_ID:
       return 0x8086;
    case PIPE_CAP_DEVICE_ID:
@@ -284,7 +478,8 @@ i915_get_param(struct pipe_screen *screen, enum pipe_cap cap)
        * assume that there's some fragmentation, and we start doing extra
        * flushing, etc.  That's the big cliff apps will care about.
        */
-      const int gpu_mappable_megabytes = is->iws->aperture_size(is->iws) * 3 / 4;
+      const int gpu_mappable_megabytes =
+         is->iws->aperture_size(is->iws) * 3 / 4;
       uint64_t system_memory;
 
       if (!os_get_total_physical_memory(&system_memory))
@@ -303,15 +498,25 @@ i915_get_param(struct pipe_screen *screen, enum pipe_cap cap)
 static float
 i915_get_paramf(struct pipe_screen *screen, enum pipe_capf cap)
 {
-   switch(cap) {
+   switch (cap) {
+   case PIPE_CAPF_MIN_LINE_WIDTH:
+   case PIPE_CAPF_MIN_LINE_WIDTH_AA:
+   case PIPE_CAPF_MIN_POINT_SIZE:
+   case PIPE_CAPF_MIN_POINT_SIZE_AA:
+      return 1;
+
+   case PIPE_CAPF_POINT_SIZE_GRANULARITY:
+   case PIPE_CAPF_LINE_WIDTH_GRANULARITY:
+      return 0.1;
+
    case PIPE_CAPF_MAX_LINE_WIDTH:
-      /* fall-through */
+      FALLTHROUGH;
    case PIPE_CAPF_MAX_LINE_WIDTH_AA:
       return 7.5;
 
-   case PIPE_CAPF_MAX_POINT_WIDTH:
-      /* fall-through */
-   case PIPE_CAPF_MAX_POINT_WIDTH_AA:
+   case PIPE_CAPF_MAX_POINT_SIZE:
+      FALLTHROUGH;
+   case PIPE_CAPF_MAX_POINT_SIZE_AA:
       return 255.0;
 
    case PIPE_CAPF_MAX_TEXTURE_ANISOTROPY:
@@ -321,9 +526,9 @@ i915_get_paramf(struct pipe_screen *screen, enum pipe_capf cap)
       return 16.0;
 
    case PIPE_CAPF_MIN_CONSERVATIVE_RASTER_DILATE:
-      /* fall-through */
+      FALLTHROUGH;
    case PIPE_CAPF_MAX_CONSERVATIVE_RASTER_DILATE:
-      /* fall-through */
+      FALLTHROUGH;
    case PIPE_CAPF_CONSERVATIVE_RASTER_DILATE_GRANULARITY:
       return 0.0f;
 
@@ -334,62 +539,42 @@ i915_get_paramf(struct pipe_screen *screen, enum pipe_capf cap)
 }
 
 bool
-i915_is_format_supported(struct pipe_screen *screen,
-                         enum pipe_format format,
-                         enum pipe_texture_target target,
-                         unsigned sample_count,
-                         unsigned storage_sample_count,
-                         unsigned tex_usage)
+i915_is_format_supported(struct pipe_screen *screen, enum pipe_format format,
+                         enum pipe_texture_target target, unsigned sample_count,
+                         unsigned storage_sample_count, unsigned tex_usage)
 {
    static const enum pipe_format tex_supported[] = {
-      PIPE_FORMAT_B8G8R8A8_UNORM,
-      PIPE_FORMAT_B8G8R8A8_SRGB,
-      PIPE_FORMAT_B8G8R8X8_UNORM,
-      PIPE_FORMAT_R8G8B8A8_UNORM,
-      PIPE_FORMAT_R8G8B8X8_UNORM,
-      PIPE_FORMAT_B4G4R4A4_UNORM,
-      PIPE_FORMAT_B5G6R5_UNORM,
-      PIPE_FORMAT_B5G5R5A1_UNORM,
-      PIPE_FORMAT_B10G10R10A2_UNORM,
-      PIPE_FORMAT_L8_UNORM,
-      PIPE_FORMAT_A8_UNORM,
-      PIPE_FORMAT_I8_UNORM,
-      PIPE_FORMAT_L8A8_UNORM,
-      PIPE_FORMAT_UYVY,
+      PIPE_FORMAT_B8G8R8A8_UNORM, PIPE_FORMAT_B8G8R8A8_SRGB,
+      PIPE_FORMAT_B8G8R8X8_UNORM, PIPE_FORMAT_R8G8B8A8_UNORM,
+      PIPE_FORMAT_R8G8B8X8_UNORM, PIPE_FORMAT_B4G4R4A4_UNORM,
+      PIPE_FORMAT_B5G6R5_UNORM, PIPE_FORMAT_B5G5R5A1_UNORM,
+      PIPE_FORMAT_B10G10R10A2_UNORM, PIPE_FORMAT_L8_UNORM, PIPE_FORMAT_A8_UNORM,
+      PIPE_FORMAT_I8_UNORM, PIPE_FORMAT_L8A8_UNORM, PIPE_FORMAT_UYVY,
       PIPE_FORMAT_YUYV,
       /* XXX why not?
       PIPE_FORMAT_Z16_UNORM, */
-      PIPE_FORMAT_DXT1_RGB,
-      PIPE_FORMAT_DXT1_RGBA,
-      PIPE_FORMAT_DXT3_RGBA,
-      PIPE_FORMAT_DXT5_RGBA,
-      PIPE_FORMAT_Z24X8_UNORM,
-      PIPE_FORMAT_Z24_UNORM_S8_UINT,
-      PIPE_FORMAT_NONE  /* list terminator */
+      PIPE_FORMAT_DXT1_RGB, PIPE_FORMAT_DXT1_SRGB, PIPE_FORMAT_DXT1_RGBA,
+      PIPE_FORMAT_DXT1_SRGBA, PIPE_FORMAT_DXT3_RGBA, PIPE_FORMAT_DXT3_SRGBA,
+      PIPE_FORMAT_DXT5_RGBA, PIPE_FORMAT_DXT5_SRGBA, PIPE_FORMAT_Z24X8_UNORM,
+      PIPE_FORMAT_FXT1_RGB, PIPE_FORMAT_FXT1_RGBA,
+      PIPE_FORMAT_Z24_UNORM_S8_UINT, PIPE_FORMAT_NONE /* list terminator */
    };
    static const enum pipe_format render_supported[] = {
-      PIPE_FORMAT_B8G8R8A8_UNORM,
-      PIPE_FORMAT_B8G8R8X8_UNORM,
-      PIPE_FORMAT_R8G8B8A8_UNORM,
-      PIPE_FORMAT_R8G8B8X8_UNORM,
-      PIPE_FORMAT_B5G6R5_UNORM,
-      PIPE_FORMAT_B5G5R5A1_UNORM,
-      PIPE_FORMAT_B4G4R4A4_UNORM,
-      PIPE_FORMAT_B10G10R10A2_UNORM,
-      PIPE_FORMAT_L8_UNORM,
-      PIPE_FORMAT_A8_UNORM,
-      PIPE_FORMAT_I8_UNORM,
-      PIPE_FORMAT_NONE  /* list terminator */
+      PIPE_FORMAT_B8G8R8A8_UNORM, PIPE_FORMAT_B8G8R8X8_UNORM,
+      PIPE_FORMAT_R8G8B8A8_UNORM, PIPE_FORMAT_R8G8B8X8_UNORM,
+      PIPE_FORMAT_B5G6R5_UNORM,   PIPE_FORMAT_B5G5R5A1_UNORM,
+      PIPE_FORMAT_B4G4R4A4_UNORM, PIPE_FORMAT_B10G10R10A2_UNORM,
+      PIPE_FORMAT_L8_UNORM,       PIPE_FORMAT_A8_UNORM,
+      PIPE_FORMAT_I8_UNORM,       PIPE_FORMAT_NONE /* list terminator */
    };
    static const enum pipe_format depth_supported[] = {
       /* XXX why not?
       PIPE_FORMAT_Z16_UNORM, */
-      PIPE_FORMAT_Z24X8_UNORM,
-      PIPE_FORMAT_Z24_UNORM_S8_UINT,
-      PIPE_FORMAT_NONE  /* list terminator */
+      PIPE_FORMAT_Z24X8_UNORM, PIPE_FORMAT_Z24_UNORM_S8_UINT,
+      PIPE_FORMAT_NONE /* list terminator */
    };
    const enum pipe_format *list;
-   uint i;
+   uint32_t i;
 
    if (sample_count > 1)
       return false;
@@ -397,7 +582,7 @@ i915_is_format_supported(struct pipe_screen *screen,
    if (MAX2(1, sample_count) != MAX2(1, storage_sample_count))
       return false;
 
-   if(tex_usage & PIPE_BIND_DEPTH_STENCIL)
+   if (tex_usage & PIPE_BIND_DEPTH_STENCIL)
       list = depth_supported;
    else if (tex_usage & PIPE_BIND_RENDER_TARGET)
       list = render_supported;
@@ -414,15 +599,12 @@ i915_is_format_supported(struct pipe_screen *screen,
    return false;
 }
 
-
 /*
  * Fence functions
  */
 
-
 static void
-i915_fence_reference(struct pipe_screen *screen,
-                     struct pipe_fence_handle **ptr,
+i915_fence_reference(struct pipe_screen *screen, struct pipe_fence_handle **ptr,
                      struct pipe_fence_handle *fence)
 {
    struct i915_screen *is = i915_screen(screen);
@@ -431,10 +613,8 @@ i915_fence_reference(struct pipe_screen *screen,
 }
 
 static bool
-i915_fence_finish(struct pipe_screen *screen,
-                  struct pipe_context *ctx,
-                  struct pipe_fence_handle *fence,
-                  uint64_t timeout)
+i915_fence_finish(struct pipe_screen *screen, struct pipe_context *ctx,
+                  struct pipe_fence_handle *fence, uint64_t timeout)
 {
    struct i915_screen *is = i915_screen(screen);
 
@@ -444,18 +624,14 @@ i915_fence_finish(struct pipe_screen *screen,
    return is->iws->fence_finish(is->iws, fence) == 1;
 }
 
-
 /*
  * Generic functions
  */
 
-
 static void
-i915_flush_frontbuffer(struct pipe_screen *screen,
-                       struct pipe_context *pipe,
-                       struct pipe_resource *resource,
-                       unsigned level, unsigned layer,
-                       void *winsys_drawable_handle,
+i915_flush_frontbuffer(struct pipe_screen *screen, struct pipe_context *pipe,
+                       struct pipe_resource *resource, unsigned level,
+                       unsigned layer, void *winsys_drawable_handle,
                        struct pipe_box *sub_box)
 {
    /* XXX: Dummy right now. */
@@ -493,7 +669,7 @@ i915_screen_create(struct i915_winsys *iws)
    switch (iws->pci_id) {
    case PCI_CHIP_I915_G:
    case PCI_CHIP_I915_GM:
-      is->is_i945 = FALSE;
+      is->is_i945 = false;
       break;
 
    case PCI_CHIP_I945_G:
@@ -504,11 +680,11 @@ i915_screen_create(struct i915_winsys *iws)
    case PCI_CHIP_Q35_G:
    case PCI_CHIP_PINEVIEW_G:
    case PCI_CHIP_PINEVIEW_M:
-      is->is_i945 = TRUE;
+      is->is_i945 = true;
       break;
 
    default:
-      debug_printf("%s: unknown pci id 0x%x, cannot create screen\n", 
+      debug_printf("%s: unknown pci id 0x%x, cannot create screen\n",
                    __FUNCTION__, iws->pci_id);
       FREE(is);
       return NULL;
@@ -525,6 +701,8 @@ i915_screen_create(struct i915_winsys *iws)
    is->base.get_param = i915_get_param;
    is->base.get_shader_param = i915_get_shader_param;
    is->base.get_paramf = i915_get_paramf;
+   is->base.get_compiler_options = i915_get_compiler_options;
+   is->base.finalize_nir = i915_finalize_nir;
    is->base.is_format_supported = i915_is_format_supported;
 
    is->base.context_create = i915_create_context;
