@@ -30,7 +30,6 @@
 #include "pan_context.h"
 #include "pan_shader.h"
 #include "pan_util.h"
-#include "panfrost-quirks.h"
 
 #include "compiler/nir/nir.h"
 #include "nir/tgsi_to_nir.h"
@@ -77,7 +76,7 @@ panfrost_shader_compile(struct pipe_screen *pscreen,
         struct util_dynarray binary;
 
         util_dynarray_init(&binary, NULL);
-        pan_shader_compile(dev, s, &inputs, &binary, &state->info);
+        screen->vtbl.compile_shader(s, &inputs, &binary, &state->info);
 
         if (binary.size) {
                 state->bin = panfrost_pool_take_ref(shader_pool,
@@ -89,7 +88,7 @@ panfrost_shader_compile(struct pipe_screen *pscreen,
         /* Don't upload RSD for fragment shaders since they need draw-time
          * merging for e.g. depth/stencil/alpha */
         bool upload = stage != MESA_SHADER_FRAGMENT;
-        screen->vtbl.prepare_rsd(dev, state, desc_pool, upload);
+        screen->vtbl.prepare_rsd(state, desc_pool, upload);
 
         panfrost_analyze_sysvals(state);
 
