@@ -341,7 +341,7 @@ svga_delete_sampler_state(struct pipe_context *pipe, void *sampler)
 
    if (svga_have_vgpu10(svga)) {
       unsigned i;
-      for (i = 0; i < 2; i++) {
+      for (i = 0; i < ARRAY_SIZE(ss->id); i++) {
          if (ss->id[i] != SVGA3D_INVALID_ID) {
             svga_hwtnl_flush_retry(svga);
 
@@ -414,6 +414,8 @@ svga_set_sampler_views(struct pipe_context *pipe,
                        enum pipe_shader_type shader,
                        unsigned start,
                        unsigned num,
+                       unsigned unbind_num_trailing_slots,
+                       bool take_ownership,
                        struct pipe_sampler_view **views)
 {
    struct svga_context *svga = svga_context(pipe);
@@ -426,8 +428,13 @@ svga_set_sampler_views(struct pipe_context *pipe,
    assert(start + num <= ARRAY_SIZE(svga->curr.sampler_views[shader]));
 
    /* Pre-VGPU10 only supports FS textures */
-   if (!svga_have_vgpu10(svga) && shader != PIPE_SHADER_FRAGMENT)
+   if (!svga_have_vgpu10(svga) && shader != PIPE_SHADER_FRAGMENT) {
+      for (unsigned i = 0; i < num; i++) {
+         struct pipe_sampler_view *view = views[i];
+         pipe_sampler_view_reference(&view, NULL);
+      }
       return;
+   }
 
    SVGA_STATS_TIME_PUSH(svga_sws(svga), SVGA_STATS_TIME_SETSAMPLERVIEWS);
 
@@ -447,10 +454,15 @@ svga_set_sampler_views(struct pipe_context *pipe,
    for (i = 0; i < num; i++) {
       enum pipe_texture_target target;
 
-      if (svga->curr.sampler_views[shader][start + i] != views[i]) {
+      any_change |= svga->curr.sampler_views[shader][start + i] != views[i];
+
+      if (take_ownership) {
+         pipe_sampler_view_reference(&svga->curr.sampler_views[shader][start + i],
+               NULL);
+         svga->curr.sampler_views[shader][start + i] = views[i];
+      } else if (svga->curr.sampler_views[shader][start + i] != views[i]) {
          pipe_sampler_view_reference(&svga->curr.sampler_views[shader][start + i],
                                      views[i]);
-         any_change = TRUE;
       }
 
       if (!views[i])
@@ -475,6 +487,14 @@ svga_set_sampler_views(struct pipe_context *pipe,
       }
    }
 
+   for (; i < num + unbind_num_trailing_slots; i++) {
+      if (svga->curr.sampler_views[shader][start + i]) {
+         pipe_sampler_view_reference(&svga->curr.sampler_views[shader][start + i],
+                                     NULL);
+         any_change = TRUE;
+      }
+   }
+
    if (!any_change) {
       goto done;
    }
@@ -517,7 +537,7 @@ svga_cleanup_sampler_state(struct svga_context *svga)
 {
    enum pipe_shader_type shader;
 
-   for (shader = 0; shader <= PIPE_SHADER_TESS_EVAL; shader++) {
+   for (shader = 0; shader <= PIPE_SHADER_COMPUTE; shader++) {
       unsigned i;
 
       for (i = 0; i < svga->state.hw_draw.num_sampler_views[shader]; i++) {
