@@ -886,7 +886,9 @@ present( struct NineSwapChain9 *This,
         handle_draw_cursor_and_hud(This, resource);
 
     fence = NULL;
-    pipe->flush(pipe, &fence, PIPE_FLUSH_END_OF_FRAME);
+    /* When threadpool is enabled, we don't submit before the fence
+     * tells us rendering was finished, thus we can flush async there */
+    pipe->flush(pipe, &fence, PIPE_FLUSH_END_OF_FRAME | (This->enable_threadpool ? PIPE_FLUSH_ASYNC : 0));
 
     /* Present now for thread_submit, because we have the fence.
      * It's possible we return WASSTILLDRAWING and still Present,
@@ -930,6 +932,8 @@ bypass_rendering:
         if (FAILED(hr)) { UNTESTED(3);return hr; }
     }
 
+    This->base.device->end_scene_since_present = 0;
+    This->base.device->frame_count++;
     return D3D_OK;
 }
 
@@ -1312,6 +1316,9 @@ NineSwapChain9_GetBackBufferCountForParams( struct NineSwapChain9 *This,
              * the Xserver will hold 4 buffers. */
             else if (!This->actx->thread_submit && count < 5)
                 count = 5;
+            /* Somehow this cases needs 5 with thread_submit, or else you get a small performance hit */
+            if (This->actx->tearfree_discard && count < 5)
+                count = 5;
         }
     }
 
