@@ -99,7 +99,7 @@ index_to_reg(compiler_context *ctx, struct lcra_state *l, unsigned reg, unsigned
         /* Report that we actually use this register, and return it */
 
         if (r.reg < 16)
-                ctx->work_registers = MAX2(ctx->work_registers, r.reg);
+                ctx->info->work_reg_count = MAX2(ctx->info->work_reg_count, r.reg + 1);
 
         return r;
 }
@@ -212,6 +212,9 @@ mir_lower_special_reads(compiler_context *ctx)
                         mark_node_class(texr, ins->src[2]);
                         mark_node_class(texw, ins->dest);
                         break;
+
+                default:
+                        break;
                 }
         }
 
@@ -321,7 +324,7 @@ mir_compute_interference(
 
         /* We need to force r1.w live throughout a blend shader */
 
-        if (ctx->is_blend) {
+        if (ctx->inputs->is_blend) {
                 unsigned r1w = ~0;
 
                 mir_foreach_block(ctx, _block) {
@@ -356,11 +359,27 @@ mir_compute_interference(
                         unsigned dest = ins->dest;
 
                         if (dest < ctx->temp_count) {
-                                for (unsigned i = 0; i < ctx->temp_count; ++i)
+                                for (unsigned i = 0; i < ctx->temp_count; ++i) {
                                         if (live[i]) {
                                                 unsigned mask = mir_bytemask(ins);
                                                 lcra_add_node_interference(l, dest, mask, i, live[i]);
                                         }
+                                }
+                        }
+
+                        /* Add blend shader interference: blend shaders might
+                         * clobber r0-r3. */
+                        if (ins->compact_branch && ins->writeout) {
+                                for (unsigned i = 0; i < ctx->temp_count; ++i) {
+                                        if (!live[i])
+                                                continue;
+
+                                        for (unsigned j = 0; j < 4; j++) {
+                                                lcra_add_node_interference(l, ctx->temp_count + j,
+                                                                0xFFFF,
+                                                                i, live[i]);
+                                        }
+                                }
                         }
 
                         /* Update live_in */
@@ -391,22 +410,22 @@ mir_is_64(midgard_instruction *ins)
 static struct lcra_state *
 allocate_registers(compiler_context *ctx, bool *spilled)
 {
-        /* The number of vec4 work registers available depends on when the
-         * uniforms start and the shader stage. By ABI we limit blend shaders
-         * to 8 registers, should be lower XXX */
-        int work_count = ctx->is_blend ? 8 :
-                16 - MAX2((ctx->uniform_cutoff - 8), 0);
+        /* The number of vec4 work registers available depends on the number of
+         * register-mapped uniforms and the shader stage. By ABI we limit blend
+         * shaders to 8 registers, should be lower XXX */
+        int rmu = ctx->info->push.count / 4;
+        int work_count = ctx->inputs->is_blend ? 8 : 16 - MAX2(rmu - 8, 0);
 
        /* No register allocation to do with no SSA */
 
         if (!ctx->temp_count)
                 return NULL;
 
-        /* Initialize LCRA. Allocate an extra node at the end for a precoloured
-         * r1 for interference */
+        /* Initialize LCRA. Allocate extra node at the end for r1-r3 for
+         * interference */
 
-        struct lcra_state *l = lcra_alloc_equations(ctx->temp_count + 1, 5);
-        unsigned node_r1 = ctx->temp_count;
+        struct lcra_state *l = lcra_alloc_equations(ctx->temp_count + 4, 5);
+        unsigned node_r1 = ctx->temp_count + 1;
 
         /* Starts of classes, in bytes */
         l->class_start[REG_CLASS_WORK]  = 16 * 0;
@@ -447,7 +466,7 @@ allocate_registers(compiler_context *ctx, bool *spilled)
                                 unsigned s = ins->src[v];
 
                                 if (s < ctx->temp_count)
-                                        min_alignment[s] = 3;
+                                        min_alignment[s] = MAX2(3, min_alignment[s]);
                         }
                 }
 
@@ -457,7 +476,7 @@ allocate_registers(compiler_context *ctx, bool *spilled)
                                 unsigned size = nir_alu_type_get_type_size(ins->src_types[v]);
 
                                 if (s < ctx->temp_count)
-                                        min_alignment[s] = (size == 64) ? 3 : 2;
+                                        min_alignment[s] = MAX2((size == 64) ? 3 : 2, min_alignment[s]);
                         }
                 }
 
@@ -480,13 +499,14 @@ allocate_registers(compiler_context *ctx, bool *spilled)
                 found_class[dest] = MAX2(found_class[dest], bytes);
 
                 min_alignment[dest] =
-                        (size == 16) ? 1 : /* (1 << 1) = 2-byte */
-                        (size == 32) ? 2 : /* (1 << 2) = 4-byte */
-                        (size == 64) ? 3 : /* (1 << 3) = 8-byte */
-                        3; /* 8-bit todo */
+                        MAX2(min_alignment[dest],
+                             (size == 16) ? 1 : /* (1 << 1) = 2-byte */
+                             (size == 32) ? 2 : /* (1 << 2) = 4-byte */
+                             (size == 64) ? 3 : /* (1 << 3) = 8-byte */
+                             3); /* 8-bit todo */
 
                 /* We can't cross xy/zw boundaries. TODO: vec8 can */
-                if (size == 16)
+                if (size == 16 && min_alignment[dest] != 4)
                         min_bound[dest] = 8;
 
                 mir_foreach_src(ins, s) {
@@ -597,7 +617,8 @@ allocate_registers(compiler_context *ctx, bool *spilled)
          * the following segment. We model this as interference.
          */
 
-        l->solutions[node_r1] = (16 * 1);
+        for (unsigned i = 0; i < 4; ++i)
+                l->solutions[ctx->temp_count + i] = (16 * i);
 
         mir_foreach_block(ctx, _blk) {
                 midgard_block *blk = (midgard_block *) _blk;
@@ -646,7 +667,7 @@ allocate_registers(compiler_context *ctx, bool *spilled)
         if (ctx->blend_src1 != ~0) {
                 assert(ctx->blend_src1 < ctx->temp_count);
                 l->solutions[ctx->blend_src1] = (16 * 2);
-                ctx->work_registers = MAX2(ctx->work_registers, 2);
+                ctx->info->work_reg_count = MAX2(ctx->info->work_reg_count, 3);
         }
 
         mir_compute_interference(ctx, l);
@@ -744,7 +765,7 @@ install_registers_instr(
         }
 
         case TAG_TEXTURE_4: {
-                if (ins->op == TEXTURE_OP_BARRIER)
+                if (ins->op == midgard_tex_op_barrier)
                         break;
 
                 /* Grab RA results */
@@ -827,7 +848,7 @@ mir_spill_register(
                 unsigned spill_class,
                 unsigned *spill_count)
 {
-        if (spill_class == REG_CLASS_WORK && ctx->is_blend)
+        if (spill_class == REG_CLASS_WORK && ctx->inputs->is_blend)
                 unreachable("Blend shader spilling is currently unimplemented");
 
         unsigned spill_index = ctx->temp_count;
@@ -842,6 +863,19 @@ mir_spill_register(
         /* Allocate TLS slot (maybe) */
         unsigned spill_slot = !is_special ? (*spill_count)++ : 0;
 
+        /* For special reads, figure out how many bytes we need */
+        unsigned read_bytemask = 0;
+
+        /* If multiple instructions write to this destination, we'll have to
+         * fill from TLS before writing */
+        unsigned write_count = 0;
+
+        mir_foreach_instr_global_safe(ctx, ins) {
+                read_bytemask |= mir_bytemask_of_read_components(ins, spill_node);
+                if (ins->dest == spill_node)
+                        ++write_count;
+        }
+
         /* For TLS, replace all stores to the spilled node. For
          * special reads, just keep as-is; the class will be demoted
          * implicitly. For special writes, spill to a work register */
@@ -850,13 +884,16 @@ mir_spill_register(
                 if (is_special_w)
                         spill_slot = spill_index++;
 
+                unsigned last_id = ~0;
+                unsigned last_fill = ~0;
+                unsigned last_spill_index = ~0;
+                midgard_instruction *last_spill = NULL;
+
                 mir_foreach_block(ctx, _block) {
                 midgard_block *block = (midgard_block *) _block;
                 mir_foreach_instr_in_block_safe(block, ins) {
                         if (ins->dest != spill_node) continue;
 
-                        midgard_instruction st;
-
                         /* Note: it's important to match the mask of the spill
                          * with the mask of the instruction whose destination
                          * we're spilling, or otherwise we'll read invalid
@@ -864,34 +901,82 @@ mir_spill_register(
                          */
 
                         if (is_special_w) {
-                                st = v_mov(spill_node, spill_slot);
+                                midgard_instruction st = v_mov(spill_node, spill_slot);
                                 st.no_spill |= (1 << spill_class);
                                 st.mask = ins->mask;
                                 st.dest_type = st.src_types[1] = ins->dest_type;
+
+                                /* Hint: don't rewrite this node */
+                                st.hint = true;
+
+                                mir_insert_instruction_after_scheduled(ctx, block, ins, st);
                         } else {
-                                ins->dest = spill_index++;
+                                unsigned bundle = ins->bundle_id;
+                                unsigned dest = (bundle == last_id)? last_spill_index : spill_index++;
+
+                                unsigned bytemask = mir_bytemask(ins);
+                                unsigned write_mask = mir_from_bytemask(mir_round_bytemask_up(
+                                                                           bytemask, 32), 32);
+
+                                if (write_count > 1 && bytemask != 0xFFFF && bundle != last_fill) {
+                                        midgard_instruction read =
+                                                v_load_store_scratch(dest, spill_slot, false, 0xF);
+                                        mir_insert_instruction_before_scheduled(ctx, block, ins, read);
+                                        write_mask = 0xF;
+                                        last_fill = bundle;
+                                }
+
+                                ins->dest = dest;
                                 ins->no_spill |= (1 << spill_class);
-                                st = v_load_store_scratch(ins->dest, spill_slot, true, ins->mask);
+
+                                bool move = false;
+
+                                /* In the same bundle, reads of the destination
+                                 * of the spilt instruction need to be direct */
+                                midgard_instruction *it = ins;
+                                while ((it = list_first_entry(&it->link, midgard_instruction, link))
+                                       && (it->bundle_id == bundle)) {
+
+                                        if (!mir_has_arg(it, spill_node)) continue;
+
+                                        mir_rewrite_index_src_single(it, spill_node, dest);
+
+                                        /* The spilt instruction will write to
+                                         * a work register for `it` to read but
+                                         * the spill needs an LD/ST register */
+                                        move = true;
+                                }
+
+                                if (move)
+                                        dest = spill_index++;
+
+                                if (last_id == bundle) {
+                                        last_spill->mask |= write_mask;
+                                        u_foreach_bit(c, write_mask)
+                                                last_spill->swizzle[0][c] = c;
+                                } else {
+                                        midgard_instruction st =
+                                                v_load_store_scratch(dest, spill_slot, true, write_mask);
+                                        last_spill = mir_insert_instruction_after_scheduled(ctx, block, ins, st);
+                                }
+
+                                if (move) {
+                                        midgard_instruction mv = v_mov(ins->dest, dest);
+                                        mv.no_spill |= (1 << spill_class);
+
+                                        mir_insert_instruction_after_scheduled(ctx, block, ins, mv);
+                                }
+
+                                last_id = bundle;
+                                last_spill_index = ins->dest;
                         }
 
-                        /* Hint: don't rewrite this node */
-                        st.hint = true;
-
-                        mir_insert_instruction_after_scheduled(ctx, block, ins, st);
-
                         if (!is_special)
                                 ctx->spills++;
                 }
                 }
         }
 
-        /* For special reads, figure out how many bytes we need */
-        unsigned read_bytemask = 0;
-
-        mir_foreach_instr_global_safe(ctx, ins) {
-                read_bytemask |= mir_bytemask_of_read_components(ins, spill_node);
-        }
-
         /* Insert a load from TLS before the first consecutive
          * use of the node, rewriting to use spilled indices to
          * break up the live range. Or, for special, insert a
@@ -959,14 +1044,13 @@ mir_spill_register(
 static void
 mir_demote_uniforms(compiler_context *ctx, unsigned new_cutoff)
 {
-        unsigned old_work_count = 16 - MAX2((ctx->uniform_cutoff - 8), 0);
+        unsigned uniforms = ctx->info->push.count / 4;
+        unsigned old_work_count = 16 - MAX2(uniforms - 8, 0);
         unsigned work_count = 16 - MAX2((new_cutoff - 8), 0);
 
         unsigned min_demote = SSA_FIXED_REGISTER(old_work_count);
         unsigned max_demote = SSA_FIXED_REGISTER(work_count);
 
-        ctx->uniform_cutoff = new_cutoff;
-
         mir_foreach_block(ctx, _block) {
                 midgard_block *block = (midgard_block *) _block;
                 mir_foreach_instr_in_block(block, ins) {
@@ -977,6 +1061,10 @@ mir_demote_uniforms(compiler_context *ctx, unsigned new_cutoff)
                                 midgard_instruction *before = ins;
 
                                 unsigned temp = make_compiler_temp(ctx);
+                                unsigned idx = (23 - SSA_REG_FROM_FIXED(ins->src[i])) * 4;
+                                assert(idx < ctx->info->push.count);
+
+                                ctx->ubo_mask |= BITSET_BIT(ctx->info->push.words[idx].ubo);
 
                                 midgard_instruction ld = {
                                         .type = TAG_LOAD_STORE_4,
@@ -985,13 +1073,15 @@ mir_demote_uniforms(compiler_context *ctx, unsigned new_cutoff)
                                         .dest_type = ins->src_types[i],
                                         .src = { ~0, ~0, ~0, ~0 },
                                         .swizzle = SWIZZLE_IDENTITY_4,
-                                        .op = midgard_op_ld_ubo_int4,
+                                        .op = midgard_op_ld_ubo_128,
                                         .load_store = {
-                                                .arg_2 = 0x1E,
+                                                .index_reg = REGISTER_LDST_ZERO,
                                         },
+                                        .constants.u32[0] = ctx->info->push.words[idx].offset
                                 };
 
-                                ld.constants.u32[0] = (23 - SSA_REG_FROM_FIXED(ins->src[i])) * 16;
+                                midgard_pack_ubo_index_imm(&ld.load_store,
+                                                           ctx->info->push.words[idx].ubo);
 
                                 mir_insert_instruction_before_scheduled(ctx, block, before, ld);
 
@@ -999,6 +1089,8 @@ mir_demote_uniforms(compiler_context *ctx, unsigned new_cutoff)
                         }
                 }
         }
+
+        ctx->info->push.count = MIN2(ctx->info->push.count, new_cutoff * 4);
 }
 
 /* Run register allocation in a loop, spilling until we succeed */
@@ -1011,7 +1103,7 @@ mir_ra(compiler_context *ctx)
         int iter_count = 1000; /* max iterations */
 
         /* Number of 128-bit slots in memory we've spilled into */
-        unsigned spill_count = DIV_ROUND_UP(ctx->tls_size, 16);
+        unsigned spill_count = DIV_ROUND_UP(ctx->info->tls_size, 16);
 
 
         mir_create_pipeline_registers(ctx);
@@ -1019,13 +1111,12 @@ mir_ra(compiler_context *ctx)
         do {
                 if (spilled) {
                         signed spill_node = mir_choose_spill_node(ctx, l);
+                        unsigned uniforms = ctx->info->push.count / 4;
 
                         /* It's a lot cheaper to demote uniforms to get more
                          * work registers than to spill to TLS. */
-                        if (l->spill_class == REG_CLASS_WORK &&
-                            ctx->uniform_cutoff > 8) {
-
-                                mir_demote_uniforms(ctx, MAX2(ctx->uniform_cutoff - 4, 8));
+                        if (l->spill_class == REG_CLASS_WORK && uniforms > 8) {
+                                mir_demote_uniforms(ctx, MAX2(uniforms - 4, 8));
                         } else if (spill_node == -1) {
                                 fprintf(stderr, "ERROR: Failed to choose spill node\n");
                                 lcra_free(l);
@@ -1054,7 +1145,7 @@ mir_ra(compiler_context *ctx)
         /* Report spilling information. spill_count is in 128-bit slots (vec4 x
          * fp32), but tls_size is in bytes, so multiply by 16 */
 
-        ctx->tls_size = spill_count * 16;
+        ctx->info->tls_size = spill_count * 16;
 
         install_registers(ctx, l);
 
